from .async_client import AsyncPypiTokenClientSession, async_pypi_token_client
from .common import (
    AllProjects,
    LoginError,
    PasswordError,
    SingleProject,
    TokenListEntry,
    TokenScope,
    TooManyAttemptsError,
    UsernameError,
)
from .credentials import PypiCredentials

__all__ = [
    "async_pypi_token_client",
    "AsyncPypiTokenClientSession",
    "PypiCredentials",
    "LoginError",
    "UsernameError",
    "PasswordError",
    "TooManyAttemptsError",
    "TokenScope",
    "AllProjects",
    "SingleProject",
    "TokenListEntry",
]



from dataclasses import dataclass
from pathlib import Path

import typer

from pypi_token_client.common import AllProjects, SingleProject

from .app import App

cli_app = typer.Typer(
    context_settings={
        "auto_envvar_prefix": "PYPITOKENCLIENT",
    }
)


@dataclass
class TyperState:
    headless: bool
    persist_to: Path | None
    username: str | None
    password: str | None
    pypi_base_url: str = "https://pypi.org"


def _app_from_typer_state(state: TyperState) -> App:
    return App(
        state.headless,
        state.persist_to,
        state.username,
        state.password,
        state.pypi_base_url,
    )


@cli_app.callback()
def typer_callback(
    ctx: typer.Context,
    headless: bool = typer.Option(
        True,
        help="whether to run the broswer in headless mode "
        "(note that in non-headless mode, it will wait for you to inspect "
        "the situation and close it manually when an error happens)",
    ),
    persist_to: str = typer.Option(
        None,
        "--persist",
        metavar="PATH",
        help="persist browser state to directory (no persistence if not set)",
    ),
    username: str = typer.Option(
        None,
        "--username",
        "-u",
        help="PyPI username (will prompt for it if not given)",
    ),
    password: str = typer.Option(
        None,
        help="PyPI password "
        "(will prompt for it or load it from the keyring if not given); "
        "you should probably NOT set this via the CLI because it "
        "will then be visible in the list of processes; "
        "it's safer to provide it as an env var",
    ),
    pypi_base_url: str = typer.Option(
        "https://pypi.org", help="base URL of the pypi website to use"
    ),
):
    ctx.obj = TyperState(
        headless,
        Path(persist_to) if persist_to is not None else None,
        username,
        password,
        pypi_base_url,
    )


@cli_app.command()
def create(
    ctx: typer.Context,
    token_name: str = typer.Argument(..., help="name of the token"),
    project: str = typer.Option(
        None, help="project for which to generate token"
    ),
):
    """
    Create a new token on PyPI
    """
    scope = AllProjects() if project is None else SingleProject(project)
    app = _app_from_typer_state(ctx.obj)
    app.create_token(token_name, scope)


@cli_app.command("list")
def list_tokens(ctx: typer.Context):
    """
    List tokens on PyPI
    """
    app = _app_from_typer_state(ctx.obj)
    app.list_tokens()


@cli_app.command()
def delete(
    ctx: typer.Context,
    name: str = typer.Argument(..., help="name of token to delete"),
):
    """
    Delete token on PyPI
    """
    app = _app_from_typer_state(ctx.obj)
    app.delete_token(name)


def cli_main():
    cli_app()


if __name__ == "__main__":
    cli_main()



"""
PyPI credential data structure and utilities.
"""
from dataclasses import dataclass
from getpass import getpass

import keyring

keyring_service_prefix = "pypi-token-client-cli"


@dataclass
class PypiCredentials:
    username: str
    "PyPI username"
    password: str
    "PyPI password"


def make_keyring_service_name(pypi_base_url: str) -> str:
    return f"{keyring_service_prefix} ({pypi_base_url})"


def get_credentials_from_keyring_and_prompt(
    pypi_base_url: str,
    username: str | None = None,
    password: str | None = None,
) -> tuple[PypiCredentials, bool]:
    """
    Get PyPI credentials from both the keyring and by prompting the user.

    Returns:
      Tuple of the credentials and whether the credentials were not in the
      keyring ("credentials are new" boolean).
    """
    if username is not None and password is not None:
        return (PypiCredentials(username, password), False)
    if username is None:
        username = input("pypi username: ")
    if password is not None:
        return (PypiCredentials(username, password), True)
    credentials = get_credentials_from_keyring(pypi_base_url, username)
    if credentials is None:
        password = getpass("pypi password: ")
        return (PypiCredentials(username, password), True)
    return (credentials, False)


def prompt_for_credentials(username: str | None = None) -> PypiCredentials:
    if username is None:
        username = input("pypi username: ")
    password = getpass("pypi password: ")
    return PypiCredentials(username, password)


def get_credentials_from_keyring(
    pypi_base_url: str, username: str
) -> PypiCredentials | None:
    cred = keyring.get_credential(
        make_keyring_service_name(pypi_base_url), username
    )
    if cred:
        return PypiCredentials(cred.username, cred.password)
    return None


def save_credentials_to_keyring(
    pypi_base_url: str, credentials: PypiCredentials
):
    keyring.set_password(
        make_keyring_service_name(pypi_base_url),
        credentials.username,
        credentials.password,
    )



import asyncio
from collections.abc import AsyncIterator
from contextlib import asynccontextmanager
from itertools import count
from pathlib import Path
from pprint import pprint
from traceback import print_exc

from .async_client import AsyncPypiTokenClientSession, async_pypi_token_client
from .common import PasswordError, TokenScope, UsernameError
from .credentials import (
    get_credentials_from_keyring_and_prompt,
    prompt_for_credentials,
    save_credentials_to_keyring,
)

max_login_attempts = 3


class App:
    def __init__(
        self,
        headless: bool = True,
        persist_to: Path | None = None,
        username: str | None = None,
        password: str | None = None,
        pypi_base_url: str = "https://pypi.org",
    ):
        self.headless = headless
        self.persist_to = persist_to
        self.username = username
        self.password = password
        self.pypi_base_url = pypi_base_url

    @asynccontextmanager
    async def _logged_in_session(
        self,
    ) -> AsyncIterator[AsyncPypiTokenClientSession]:
        # don't do anything interactive (e.g. ask about saving to keyring or
        # retry with prompt) if both username and password are provided
        # (generally suggests no interactivity is desired)
        interactive = self.username is None or self.password is None
        (
            credentials,
            credentials_are_new,
        ) = get_credentials_from_keyring_and_prompt(
            self.pypi_base_url, self.username, self.password
        )
        async with async_pypi_token_client(
            credentials, self.headless, self.persist_to, self.pypi_base_url
        ) as session:
            for attempt in count():
                try:
                    did_login = await session.login()
                    if did_login and credentials_are_new and interactive:
                        save = input(
                            "success! save credentials to keyring (Y/n)? "
                        )
                        if save == "Y":
                            save_credentials_to_keyring(
                                self.pypi_base_url, credentials
                            )
                            print("saved")
                        else:
                            print("not saving")
                    break
                except (UsernameError, PasswordError) as e:
                    print(f"Login failed: {e}")
                    if attempt >= max_login_attempts or not interactive:
                        print("Giving up.")
                        raise
                    credentials = prompt_for_credentials()
                    credentials_are_new = True
                    session.credentials = credentials
            yield session

    @staticmethod
    @asynccontextmanager
    async def _handle_errors(session: AsyncPypiTokenClientSession):
        try:
            yield
        except Exception:
            print_exc()
            if session.headless:
                print(
                    "If you want to see what exactly went wrong by looking at "
                    "the browser window, rerun in non-headless mode"
                )
            else:
                print_exc()
                print(
                    "check browser window for what exactly went wrong "
                    "and close it once done"
                )
                await session.wait_until_closed()
            exit(1)

    @asynccontextmanager
    async def _logged_in_error_handling_session(
        self,
    ) -> AsyncIterator[AsyncPypiTokenClientSession]:
        async with self._logged_in_session() as session, self._handle_errors(
            session
        ):
            yield session

    def create_token(self, token_name: str, scope: TokenScope) -> None:
        async def _run():
            async with self._logged_in_error_handling_session() as session:
                token = await session.create_token(token_name, scope)
            print("Created token:")
            print(token)

        asyncio.run(_run())

    def list_tokens(self) -> None:
        async def _run():
            async with self._logged_in_error_handling_session() as session:
                tokens = await session.get_token_list()
            pprint(tokens)

        asyncio.run(_run())

    def delete_token(
        self,
        name: str,
    ) -> None:
        async def _run():
            async with self._logged_in_error_handling_session() as session:
                await session.delete_token(name)

        asyncio.run(_run())



"""
`async`/`await`-based PyPI token client
"""
from asyncio import Lock
from contextlib import asynccontextmanager
from functools import wraps
from pathlib import Path
from typing import AsyncIterator, Sequence

from dateutil.parser import isoparse
from playwright.async_api import async_playwright

from .common import (
    AllProjects,
    PasswordError,
    SingleProject,
    TokenListEntry,
    TokenNameError,
    TokenScope,
    TooManyAttemptsError,
    UnexpectedContentError,
    UnexpectedPageError,
    UsernameError,
)
from .credentials import PypiCredentials
from .utils.playwright import launch_ephemeral_chromium_context
from .utils.sequences import one_or_none


def _expect_page(page, expected_url: str):
    if page.url != expected_url:
        raise UnexpectedPageError(
            f"ended up on unexpected page {page.url} (expected {expected_url})"
        )


@asynccontextmanager
async def async_pypi_token_client(
    credentials: PypiCredentials,
    headless: bool = False,
    persist_to: Path | str | None = None,
    base_url: str = "https://pypi.org",
) -> AsyncIterator["AsyncPypiTokenClientSession"]:
    """
    Context manager for launching an async client session.

    This is the main starting point for using the async client.

    Args:
        credentials: Credentials to log into PyPI with.
        headless: If true, the browser window will not be shown.
        persist_to: Directory in which to persist the browser state. ``None``
            means no persistence.
        base_url: PyPI base URL.

    Returns:
      A context manager for the async session.
    """
    async with async_playwright() as p:
        if persist_to is None:
            context = await launch_ephemeral_chromium_context(
                p, headless=headless
            )
        else:
            context = await p.chromium.launch_persistent_context(
                Path(persist_to), headless=headless
            )
        pages = context.pages
        assert len(pages) == 1
        page = pages[0]
        yield AsyncPypiTokenClientSession(
            context, page, credentials, headless, base_url
        )


def _with_lock(meth):
    @wraps(meth)
    async def _with_lock(self, *args, **kwargs):
        async with self._lock:
            return await meth(self, *args, **kwargs)

    return _with_lock


class AsyncPypiTokenClientSession:
    """
    Async token client session.

    Should not be instantiated directly but only through
    :func:`async_pypi_token_client`.

    A session's lifecycle corresponds to that of the browser instance which is
    used to perform operations on the PyPI web interface. When multiple
    operations have to be performed in sequence, it makes sense to do so in the
    a single session to minimize the number of times the browser has to be
    restarted, as this is fairly resource intensive and time consuming.
    """

    def __init__(
        self,
        context,
        page,
        credentials: PypiCredentials,
        headless: bool = True,
        base_url: str = "https://pypi.org",
    ):
        self.context = context
        self.page = page
        self.credentials = credentials
        self.headless = headless
        self.base_url = base_url
        self._lock = Lock()

    async def _get_logged_in_user(self) -> str | None:
        user_button = one_or_none(
            await self.page.locator(
                "#user-indicator > nav:first-child > button"
            ).all()
        )
        if user_button is None:
            return None
        username = (await user_button.inner_text()).strip()
        return username

    async def _handle_login(self) -> bool:
        """
        Automatically handle login if necessary, otherwise do nothing.

        Returns:
            `True` if a login was actually performed, `False` if nothing was
            done.
        """
        logged_in_user = await self._get_logged_in_user()
        if logged_in_user is not None:
            if logged_in_user == self.credentials.username:
                print("no login required")
                return False
            else:
                # TODO log out & go to login page
                raise NotImplementedError(
                    f"logged-in user {logged_in_user!r} doesn't match "
                    f"credential username {self.credentials.username!r}, "
                    "which can't be handled yet"
                )
        if not self.page.url.startswith(
            self.base_url.rstrip("/") + "/account/login/"
        ):
            print("no login required")
            return False
        username_input = one_or_none(
            await self.page.locator("#username").all()
        )
        if not username_input:
            raise UnexpectedContentError(
                "username field not found on login page"
            )
        password_input = one_or_none(
            await self.page.locator("#password").all()
        )
        if not password_input:
            raise UnexpectedContentError(
                "password field not found on login page"
            )
        await username_input.fill(self.credentials.username)
        await password_input.fill(self.credentials.password)
        async with self.page.expect_event(
            "domcontentloaded"
        ), self.page.expect_navigation():
            print("logging in...")
            await password_input.press("Enter")
        if self.page.url.startswith(
            self.base_url.rstrip("/") + "/account/login/"
        ):
            username_errors_or_none = one_or_none(
                await self.page.locator("#username-errors ul li").all()
            )
            username_error = (
                await username_errors_or_none.inner_text()
                if username_errors_or_none is not None
                else None
            )
            if username_error is not None:
                raise UsernameError(username_error)
            password_errors_or_none = one_or_none(
                await self.page.locator("#password-errors ul li").all()
            )
            password_error = (
                await password_errors_or_none.inner_text()
                if password_errors_or_none is not None
                else None
            )
            if password_error is not None:
                if "too many unsuccessful login attempts" in password_error:
                    raise TooManyAttemptsError(password_error)
                else:
                    raise PasswordError(password_error)
        return True

    async def _confirm_password(self):
        confirm_heading = one_or_none(
            await self.page.get_by_text("Confirm password to continue").all()
        )
        if not confirm_heading:
            print("no password confirmation required")
            return
        password_input = one_or_none(
            await self.page.locator("#password").all()
        )
        if not password_input:
            raise UnexpectedContentError("no password field found")
            return
        await password_input.fill(self.credentials.password)
        async with self.page.expect_event(
            "domcontentloaded"
        ), self.page.expect_navigation():
            print("confirming password...")
            await password_input.press("Enter")

    async def wait_until_closed(self):
        """
        Wait until the user closes the browser if it's not headless.

        Has no effect and returns immediately in headless mode.
        """

        await self.page.wait_for_event("close", timeout=0)

    @_with_lock
    async def create_token(self, name: str, scope: TokenScope) -> str:
        """
        Create a new token on PyPI.

        Args:
            name: Name of the token to create.
            scope: The token's desired scope.

        Returns:
            The created token.
        """
        # validate & extract from args
        if isinstance(scope, AllProjects):
            scope_selector_value = "scope:user"
        elif isinstance(scope, SingleProject):
            scope_selector_value = f"scope:project:{scope.name}"
        else:
            raise TypeError(f"invalid token scope: {scope}")
        # /validate args
        await self.page.goto(
            self.base_url + "/manage/account/token/",
            wait_until="domcontentloaded",
        )
        # login if necessary
        await self._handle_login()
        # confirm password if necessary
        await self._confirm_password()
        # fill in token name field
        name_input = one_or_none(await self.page.locator("#description").all())
        if name_input is None:
            raise UnexpectedContentError("no token name field found on page")
        await name_input.fill(name)
        # select token scope => project only
        scope_selector = one_or_none(
            await self.page.locator("#token_scope").all()
        )
        if scope_selector is None:
            raise UnexpectedContentError("no scope selector found on page")
        await scope_selector.select_option(value=scope_selector_value)
        async with self.page.expect_event(
            "domcontentloaded"
        ), self.page.expect_navigation():
            print(f"creating token {name!r}...")
            await name_input.press("Enter")
        name_errors_or_none = one_or_none(
            await self.page.locator("#token-name-errors ul li").all()
        )
        name_error = (
            await name_errors_or_none.inner_text()
            if name_errors_or_none is not None
            else None
        )
        if name_error is not None:
            raise TokenNameError(name_error)
        token_block = one_or_none(
            await self.page.locator("#provisioned-key > code").all()
        )
        if not token_block:
            raise UnexpectedContentError("no token block found on page")
        token = await token_block.inner_text()
        return token

    @_with_lock
    async def login(self) -> bool:
        """
        Log into PyPI if necessary.

        Normally, this does not need to be called explicitly as all other
        methods in this class perform logins automatically.

        One use case for this is to find out whether the given credentials are
        correct without doing anything else. It should however be noted that an
        actual login will only be performed when necessary, i.e. when the
        session's current state (resulting from loaded persistent browser state
        or prior actions) isn't already logged in.

        Returns:
            `True` if a login was actually performed, `False` if nothing was
            done.
        """
        await self.page.goto(
            self.base_url + "/account/login/",
            wait_until="domcontentloaded",
        )
        # login if necessary
        return await self._handle_login()

    @_with_lock
    async def get_token_list(self) -> Sequence[TokenListEntry]:
        """
        Get list of tokens for the logged-in account on PyPI.

        Returns:
            List of tokens.
        """
        await self.page.goto(
            self.base_url + "/manage/account/",
            wait_until="domcontentloaded",
        )
        # login if necessary
        await self._handle_login()
        # confirm password if necessary
        await self._confirm_password()
        # get list
        token_rows = await self.page.locator(
            "#api-tokens > table > tbody > tr"
        ).all()
        token_list = []
        for row in token_rows:
            cols = await row.locator("th,td").all()
            name = await cols[0].inner_text()
            scope_str = await cols[1].inner_text()
            scope = (
                AllProjects()
                if scope_str == "All projects"
                else SingleProject(scope_str)
            )
            created = isoparse(
                await one_or_none(
                    await cols[2].locator("time").all()
                ).get_attribute("datetime")
            )
            last_used_time_elem = one_or_none(
                await cols[3].locator("time").all()
            )
            last_used = (
                isoparse(await last_used_time_elem.get_attribute("datetime"))
                if last_used_time_elem is not None
                else None
            )
            entry = TokenListEntry(name, scope, created, last_used)
            token_list.append(entry)
        return token_list

    @_with_lock
    async def delete_token(self, name: str):
        """
        Delete token on PyPI.

        Args:
            name: Name of the token to delete.
        """
        await self.page.goto(
            self.base_url + "/manage/account/",
            wait_until="domcontentloaded",
        )
        # login if necessary
        await self._handle_login()
        # confirm password if necessary
        await self._confirm_password()
        # get list
        token_rows = await self.page.locator(
            "#api-tokens > table > tbody > tr"
        ).all()
        for row in token_rows:
            cols = await row.locator("th,td").all()
            listed_name = await cols[0].inner_text()
            if listed_name != name:
                continue
            options_button = one_or_none(
                await cols[4]
                .locator("nav > button")
                .get_by_text("Options", exact=True)
                .all()
            )
            if options_button is None:
                raise UnexpectedContentError(
                    "no options button found for token"
                )
            await options_button.click()
            remove_button = (
                cols[4].locator("nav a").get_by_text("Remove token")
            )
            await remove_button.wait_for(state="visible", timeout=5000)
            await remove_button.click()
            confirm_dialog_heading = self.page.get_by_text(
                f"Remove API token - {name}", exact=True
            )
            confirm_dialog = self.page.locator(
                'div[role="dialog"]', has=confirm_dialog_heading
            )
            await confirm_dialog.wait_for(state="visible", timeout=5000)
            password_input = one_or_none(
                await confirm_dialog.locator('input[type="password"]').all()
            )
            if password_input is None:
                raise UnexpectedContentError("no password field found")
            await password_input.fill(self.credentials.password)
            async with self.page.expect_event(
                "domcontentloaded"
            ), self.page.expect_navigation():
                print(f"deleting token {name!r}...")
                await password_input.press("Enter")
            await self.page.get_by_text("Deleted API token").wait_for(
                state="visible", timeout=5000
            )
            print(f"deleted token {name!r}")
            return
        else:
            print(f"no token named {name} found. nothing to do")
            return



from typing import Any, Sequence


def one_or_none(seq: Sequence[Any]):
    if len(seq) > 1:
        raise ValueError("more than one element found")
    elif len(seq) == 1:
        return seq[0]
    return None



async def launch_ephemeral_chromium_context(p, headless: bool = True):
    """
    Ephemeral version of Playwright's chromium.launch_persistent_context.

    No idea why they didn't just include that themselves...
    """
    browser = await p.chromium.launch(headless=headless)
    context = await browser.new_context()
    await context.new_page()
    return context



"""
Data structures common to both sync and async client.
"""
from dataclasses import dataclass
from datetime import datetime
from pathlib import Path

user_data_dir = str(Path("~/.autopypitok/persist-chromium").expanduser())
max_login_attempts = 3


class UnexpectedPageError(Exception):
    pass


class UnexpectedContentError(Exception):
    pass


class LoginError(Exception):
    pass


class UsernameError(LoginError):
    pass


class PasswordError(LoginError):
    pass


class TooManyAttemptsError(LoginError):
    pass


class TokenNameError(Exception):
    pass


@dataclass
class TokenScope:
    pass


@dataclass
class AllProjects(TokenScope):
    pass


@dataclass
class SingleProject(TokenScope):
    name: str
    "Name of the project for which this token should be valid"


@dataclass
class TokenListEntry:
    name: str
    scope: TokenScope
    created: datetime
    last_used: datetime | None



"""
Sync "client", which is just a bunch of wrapper functions around the async one.

TODO missing a lot of functionality, don't use yet.

NOTE that this is less flexible than the async client because you can't have
sessions that span multiple function invocations. I tried to support sessions
here too but to no avail:
https://gitlab.com/smheidrich/pypi-token-client/-/merge_requests/1
"""
import asyncio
from pathlib import Path

from pypi_token_client.common import TokenScope

from . import async_client
from .credentials import PypiCredentials


async def _create_token_async(
    token_name: str,
    scope: TokenScope,
    credentials: PypiCredentials,
    headless: bool,
    persist_to: Path | str | None,
):
    async with async_client.async_pypi_token_client(
        credentials, headless, persist_to
    ) as session:
        return await session.create_token(token_name, scope)


def create_token(
    token_name: str,
    scope: TokenScope,
    credentials: PypiCredentials,
    headless: bool = False,
    persist_to: Path | str | None = None,
):
    return asyncio.run(
        _create_token_async(
            token_name, scope, credentials, headless, persist_to
        )
    )


async def _get_token_list_async(
    credentials: PypiCredentials,
    headless: bool,
    persist_to: Path | str | None,
):
    async with async_client.async_pypi_token_client(
        credentials, headless, persist_to
    ) as session:
        return await session.get_token_list()


def get_token_list(
    credentials: PypiCredentials,
    headless: bool = False,
    persist_to: Path | str | None = None,
):
    return asyncio.run(
        _get_token_list_async(credentials, headless, persist_to)
    )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pypi_token_client', 'pypi_token_client.utils']

package_data = \
{'': ['*']}

install_requires = \
['keyring>=23.13.1,<24.0.0',
 'playwright>=1.29,<2.0',
 'python-dateutil>=2.8.2,<3.0.0',
 'typer>=0.7.0,<0.8.0']

entry_points = \
{'console_scripts': ['pypi-token-client = pypi_token_client.cli:cli_main']}

setup_kwargs = {
    'name': 'pypi-token-client',
    'version': '1.0.2',
    'description': 'Library and CLI tool for retrieving PyPI project tokens',
    'long_description': "# pypi-token-client\n\n[![pipeline status](https://gitlab.com/smheidrich/pypi-token-client/badges/main/pipeline.svg?style=flat-square)](https://gitlab.com/smheidrich/pypi-token-client/-/commits/main)\n[![docs](https://img.shields.io/badge/docs-online-brightgreen?style=flat-square)](https://smheidrich.gitlab.io/pypi-token-client/)\n[![pypi](https://img.shields.io/pypi/v/pypi-token-client)](https://pypi.org/project/pypi-token-client/)\n[![supported python versions](https://img.shields.io/pypi/pyversions/pypi-token-client)](https://pypi.org/project/pypi-token-client/)\n\nLibrary and CLI tool for retrieving PyPI project tokens.\n\n## Purpose\n\nPyPI allows the creation of per-project tokens but\n[doesn't](https://github.com/pypi/warehouse/issues/6396) currently have an API\nto do so. While integration with CI providers is\n[planned](https://github.com/pypi/warehouse/issues/6396#issuecomment-1345585291),\napparently there is\n[no plan](https://github.com/pypi/warehouse/issues/6396#issuecomment-1345667940)\nfor an API that would allow one to create tokens from a local development\nmachine.\n\nThis tool seeks to provide a client exposing this functionality anyway by\nwhatever means necessary.\n\n## Operating principle\n\nBecause there is no API and I'm also too lazy to try and figure out the exact\nsequence of HTTP requests one would have to make to simulate what happens when\nrequesting tokens on the PyPI website, for now this tool just uses\n[Playwright](https://playwright.dev/python/) to automate performing the\nnecessary steps in an *actual* browser.\n\nThis might be overkill and brittle but it works for now 🤷\n\n## Installation\n\n```bash\npip3 install pypi-token-client\n# install the necessary browsers for Playwright\nplaywright install\n```\n",
    'author': 'smheidrich',
    'author_email': 'smheidrich@weltenfunktion.de',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://smheidrich.gitlab.io/pypi-token-client/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)
